--[[
--
-- oUF_Rabbit
--
-- Custom layout for oUF.
-- Based on VuBce, rUnits, TsoHG, Lily and Lyn.
-- by rabbit.magtheridon@gmail.com
--
--]]

local statusbarTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\statusbar"
local borderTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\border"
local font = "Interface\\AddOns\\oUF_Rabbit\\media\\font.ttf"
local upperfont = "Interface\\AddOns\\oUF_Rabbit\\media\\upperfont.ttf"
local power = oUF.colors.power
local _, pClass = UnitClass("player")

local function menu(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if unit == "party" or unit == "partypet" then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif _G[cunit.."FrameDropDown"] then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local grey = {0.5, 0.5, 0.5}
local white = {1, 1, 1}
local name = "%s %s"
local classifications = {
	worldboss = "??",
	rareelite = "%s*+",
	rare = "%s*",
	elite = "%s+",
}
local function updateName(self, event, unit)
	if self.unit == unit then
		if unit == "target" then
			local cl = UnitClassification(unit)
			local level = classifications[cl] and classifications[cl]:format(UnitLevel(unit)) or UnitLevel(unit)
			local n = UnitName(unit)
			self.Name:SetText(name:format(level, n))
		else
			local n = UnitName(unit)
			if unit == "targettarget" or unit == "focus" then
				self.Name:SetText(n:sub(1, 3))
			else
				self.Name:SetText(n)
			end
		end

		local c = (not UnitIsConnected(unit) or UnitIsGhost(unit) or UnitIsDead(unit)) and grey or white
		self.Name:SetTextColor(unpack(c))
	end
end

local function short(v)
	if v <= 9999 then return v end
	if v >= 1000000 then
		local value = string.format("%.1fm", v/1000000)
		return value
	elseif v >= 10000 then
		local value = string.format("%.1fk", v/1000)
		return value
	end
end

local function updateHealth(self, event, unit, bar, min, max)
	if UnitIsDead(unit) then
		bar.value:SetText("Dead")
		bar.value:SetTextColor(.5, .5, .5)
	elseif UnitIsGhost(unit) then
		bar.value:SetText("Ghost")
		bar.value:SetTextColor(.5, .5, .5)
	elseif not UnitIsConnected(unit) then
		bar.value:SetText("Off")
		bar.value:SetTextColor(.5, .5, .5)
	else
		local value = nil
		if unit == "player" then
			if max == min then
				value = short(max)
			else
				value = "-" .. short(max - min)
			end
		elseif max == 100 then -- Percentage value
			value = min .. "%"
		elseif unit == "focus" or unit == "targettarget" then -- Percentage value
			value = math.ceil((min / max) * 100) .. "%"
		elseif min ~= max then
			value = "-" .. (max - min)
		elseif unit == "target" then
			value = max
		end
		if value then
			bar.value:SetText(value)
			if max == min then
				bar.value:SetTextColor(1, 1, 1)
			else
				bar.value:SetTextColor(1, 0.5, 0.5)
			end
		else
			bar.value:SetText("")
		end
	end
end

local defaultAuraColor = {r = 0.25, g = 0.25, b = 0.35}
local function auraIcon(self, button, icons, index, debuff)
	local color = defaultAuraColor
	if debuff and self.unit then
		local _, _, _, _, dtype = UnitDebuff(self.unit, index)
		color = DebuffTypeColor[dtype or "none"]
	end
	local t = button:CreateTexture(nil, "OVERLAY")
	t:SetTexture(borderTexture)
	t:SetAllPoints(button)
	t:SetVertexColor(color.r, color.g, color.b)
	button.icon:SetTexCoord(.07, .93, .07, .93)
end

local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
	insets = {left = -2, right = -2, top = -2, bottom = -2},
}

local function func(settings, self, unit)
	self.menu = menu
	self:EnableMouse(true)
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")

	if not unit then -- Party
		self:SetWidth(168) 
		self:SetHeight(36)
	elseif unit == "targettarget" or unit == "focus" then
		self:SetWidth(92)
		self:SetHeight(20)
	elseif unit == "player" or unit == "target" then
		self:SetWidth(192)
		self:SetHeight(36)
	elseif unit == "pet" then
		self:SetWidth(192)
		self:SetHeight(20)
	end

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, 0.9)

	local hp = CreateFrame("StatusBar")
	hp:SetHeight(20)
	hp:SetStatusBarTexture(statusbarTexture)
	hp:SetStatusBarColor(0, 1, 0)
	hp:SetParent(self)
	hp:SetPoint("TOP")
	hp:SetPoint("LEFT")
	hp:SetPoint("RIGHT")

	local hpp = hp:CreateFontString(nil, "OVERLAY")
	if unit == "player" then
		hpp:SetPoint("CENTER", 0, 0)
		hpp:SetFont(upperfont, 14, "OUTLINE")
	else
		hpp:SetPoint("RIGHT", -4, 0)
		hpp:SetFont(upperfont, 14, "OUTLINE")
	end
	hpp:SetShadowOffset(1, -1)
	hpp:SetTextColor(1, 1, 1)

	hp.value = hpp
	hp.colorDisconnected = true
	hp.colorTapping = true
	hp.colorClassNPC = true
	hp.colorClass = true
	hp.colorReaction = true

	if unit == "pet" and pClass == "HUNTER" then
		hp.colorHappiness = true
	end

	self.Health = hp
	self.PostUpdateHealth = updateHealth

	local name = hp:CreateFontString(nil, "OVERLAY")
	name:SetPoint("LEFT", 4, 0)
	name:SetPoint("RIGHT", hpp, "LEFT")
	name:SetJustifyH("LEFT")
	name:SetFont(font, 14, "OUTLINE")
	name:SetShadowOffset(1, -1)
	self.Name = name
	self.UNIT_NAME_UPDATE = updateName
	if unit == "player" then
		self.Name:Hide()
	end

	if IsAddOnLoaded("oUF_CombatFeedbackText") and unit == "player" then
		local cbft = hp:CreateFontString(nil, "OVERLAY")
		cbft:SetPoint("CENTER", self, "CENTER")
		cbft:SetFontObject(GameFontNormal)
		self.CombatFeedbackText = cbft
		self.CombatFeedbackText.maxAlpha = .6
		-- The only thing we don't ignore is heals.
		self.CombatFeedbackText.ignoreImmune = true
		self.CombatFeedbackText.ignoreDamage = true
		self.CombatFeedbackText.ignoreEnergize = true
		self.CombatFeedbackText.ignoreOther = true
	end

	local ricon = hp:CreateTexture(nil, "OVERLAY")
	ricon:SetHeight(24)
	ricon:SetWidth(24)
	ricon:SetPoint("CENTER", hp, "TOP", 0, 0)
	ricon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = ricon

	if unit ~= "targettarget" and unit ~= "focus" and unit ~= "pet" then
		local pp = CreateFrame("StatusBar")
		pp:SetHeight(8)
		hp:SetHeight(28)
		pp:SetStatusBarTexture(statusbarTexture)
		pp:SetStatusBarColor(.25, .25, .35)

		pp:SetParent(self)
		pp:SetPoint("BOTTOM", self, "BOTTOM")
		pp:SetPoint("LEFT")
		pp:SetPoint("RIGHT")

		pp.colorTapping = true
		pp.colorDisconnected = true
		pp.colorPower = true

		self.Power = pp
	end

	if not unit then
		local debuffs = CreateFrame("Frame", nil, self)
		local s = self.Power:GetHeight() + self.Health:GetHeight()
		debuffs.size = s
		debuffs:SetHeight(s)
		debuffs:SetWidth(s * 3)
		debuffs:SetPoint("LEFT", self, "RIGHT", 4, 0)
		debuffs.initialAnchor = "TOPLEFT"
		debuffs.num = 3 -- max debuffs
		self.Debuffs = debuffs
	elseif unit == "target" then
		local w = self:GetWidth() / 2

		local buffs = CreateFrame("Frame", nil, self)
		buffs.size = 22.5
		buffs:SetHeight((buffs.size + 2) * 4)
		buffs:SetWidth(w)
		buffs:SetPoint("BOTTOMLEFT", self, "TOPLEFT", -2, 4)
		buffs.numBuffs = 16
		buffs.spacing = 2
		self.Buffs = buffs

		local debuffs = CreateFrame("Frame", nil, self)
		debuffs.size = 22.5
		debuffs:SetHeight((debuffs.size + 2) * 4)
		debuffs:SetWidth(w)
		debuffs:SetPoint("BOTTOMLEFT", buffs, "BOTTOMRIGHT", 4, 0)
		debuffs.numDebuffs = 16
		debuffs.spacing = 2
		debuffs.initialAnchor = "BOTTOMRIGHT"
		debuffs["growth-x"] = "LEFT"
		debuffs.showDebuffType = true
		self.Debuffs = debuffs

		local cpoints = self:CreateFontString(nil, "OVERLAY")
		cpoints:SetPoint("RIGHT", self, "LEFT", -8, 1)
		cpoints:SetFont(DAMAGE_TEXT_FONT, 38, "OUTLINE")
		cpoints:SetTextColor(1, 1, 1)
		cpoints:SetJustifyH("RIGHT")
		self.CPoints = cpoints
	end

	if unit == "player" or unit == "target" then
		local cb = CreateFrame("StatusBar", nil, self)
		cb:SetStatusBarTexture(statusbarTexture)
		if unit == "player" then
			local color = RAID_CLASS_COLORS[pClass]
			cb:SetStatusBarColor(color.r - 0.2, color.g - 0.2, color.b - 0.2)

			cb:SetHeight(24)
			cb:SetWidth(240)

			cb:SetBackdrop({
				bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
				insets = {top = -3, left = -3, bottom = -3, right = -3}
			})

			cb.SafeZone = cb:CreateTexture(nil,"ARTWORK")
			cb.SafeZone:SetTexture(statusbarTexture)
			cb.SafeZone:SetVertexColor(.75, .10, .10, .6)
			cb.SafeZone:SetPoint("TOPRIGHT")
			cb.SafeZone:SetPoint("BOTTOMRIGHT")
			cb:SetPoint("CENTER", UIParent, "CENTER", 0, -350)
		else
			cb:SetStatusBarColor(0.80, 0.01, 0)
			cb:SetHeight(24)
			cb:SetWidth(240)

			cb:SetBackdrop({
				bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
				insets = {top = -3, left = -29, bottom = -3, right = -3}
			})

			cb.Icon = cb:CreateTexture(nil, "OVERLAY")
			cb.Icon:SetPoint("RIGHT", cb, "LEFT", -3, 0)
			cb.Icon:SetHeight(22)
			cb.Icon:SetWidth(22)
			cb.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)

			cb:SetPoint("CENTER", UIParent, "CENTER", 11, -100)
		end

		cb:SetBackdropColor(0, 0, 0, 0.5)

		cb.bg = cb:CreateTexture(nil, "BORDER")
		cb.bg:SetAllPoints(cb)
		cb.bg:SetTexture(0, 0, 0, 0.6)

		cb.Text = cb:CreateFontString(nil, "OVERLAY")
		cb.Text:SetPoint("LEFT", cb, 4, 0)
		cb.Text:SetFont(font, 11, "OUTLINE")
		cb.Text:SetShadowOffset(1, -1)
		cb.Text:SetTextColor(1, 1, 1)
		cb.Text:SetJustifyH("LEFT")
		cb.Time = cb:CreateFontString(nil, "OVERLAY")
		cb.Time:SetPoint("RIGHT", cb, -2, 0)
		cb.Time:SetFont(upperfont, 12, "OUTLINE")
		cb.Time:SetTextColor(1, 1, 1)
		cb.Time:SetJustifyH("RIGHT")

		self.Castbar = cb
	end

	if not unit then
		self.Range = true
		self.inRangeAlpha = 1
		self.outsideRangeAlpha = .5
	end

	self.PostCreateAuraIcon = auraIcon

	return self
end

oUF:RegisterStyle("Rabbit", setmetatable({}, {__call = func}))
oUF:SetActiveStyle("Rabbit")

local player = oUF:Spawn("player", "oUF_Player")
player:SetPoint("BOTTOM", -114, 160)
local pet = oUF:Spawn("pet", "oUF_Pet")
pet:SetPoint("TOPLEFT", player, "BOTTOMLEFT", 0, -8)

local target = oUF:Spawn("target", "oUF_Target")
target:SetPoint("BOTTOM", 114, 160)

local tot = oUF:Spawn("targettarget", "oUF_TargetTarget")
tot:SetPoint("TOPRIGHT", target, "BOTTOMRIGHT", 0, -8)
local focus = oUF:Spawn("focus", "oUF_Focus")
focus:SetPoint("TOPLEFT", target, "BOTTOMLEFT", 0, -8)

local party = oUF:Spawn("header", "oUF_Party")
party:SetPoint("TOPLEFT", 8, -30)
party:SetAttribute("yOffset", -8)
party:SetAttribute("showParty", true)

local toggleParty = CreateFrame("Frame")
toggleParty:SetScript("OnEvent", function()
	if GetNumRaidMembers() > 0 then
		party:Hide()
	elseif GetNumPartyMembers() > 0 then
		party:Show()
	end
end)
toggleParty:RegisterEvent("PARTY_MEMBERS_CHANGED")
toggleParty:RegisterEvent("PARTY_LEADER_CHANGED")
toggleParty:RegisterEvent("RAID_ROSTER_UPDATE")
toggleParty:RegisterEvent("PLAYER_LOGIN")

